 /***************************************************************************
                          ProjectWid.h  -  description
                             -------------------
    begin                : sam jui 5 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROJECTWID_H
#define PROJECTWID_H

#include <QWidget>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QTimer>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMenu>

#include "Project.h"
#include "PikMain.h"

/**A widget to display and edit  informations about the current project
  *@author Alain Gibaud
  */
class NewProject ;
class PikMain ;
class ProjectEditorWid ;
class ConfigBitsWizard ;

class ProjectWid : public QFrame
{
    Q_OBJECT
public:
    ProjectWid (PikMain *mainwindow, QWidget *parent, const char *);
    ~ProjectWid();
    /** Add a file name to the project tree
        (filename is absolute or relative to the project file location) */
    void addFile2Tree ( const QString&name );
    /** Returns the current project object (might be NULL) */
    Project *getProject();
    /** return the main window */
    PikMain *mainWindow() ;
    /** Drag and drop management
        */
    virtual void dragEnterEvent ( QDragEnterEvent * );
    /** Drop on project widget */
    virtual void dropEvent ( QDropEvent *ev );

    void updateCEntryPoints ( QBEditorDocument *doc , QTreeWidgetItem *it );
    void updateAsmEntryPoints ( QBEditorDocument *doc, QTreeWidgetItem  *it );

public slots:
    void slotContextMenu(const QPoint & ) ;

    // void slotProjectRButton ( QTreeWidgetItem  *, const QPoint &, int )   ;
    /** No descriptions */
    void slotCreateProject ( const QString&, const QString&, const QString& );
    /** No descriptions */
    void slotNewProjectClosed();
    /** No descriptions */
    void slotCloseProject();
    /** No descriptions */
    void slotInsertFile();
    /** No descriptions */
    void slotOpenProject();
    /** remove the current item from tree (and the file from project)  */
    void slotRemoveFile(QTreeWidgetItem *item);
    /** No descriptions */
    void slotInsertCurrentFile();
    /** Calls the Project Editor Widget */
    void slotEditProject();
    /** No descriptions */
    void slotNewProject();
    /** Updates the UI when a project is opened/closed */
    void slotProjectIsOpen ( bool open );
    /** No descriptions */
    void slotProjectEditorClosed();
    /** No descriptions */
    void slotOpenProjectFile ( const QString& filename ) ;
    virtual void slotUpdateEntryPoints();
    /** activate config bits editor */
    void slotEditConfigBits();
    /** do housekeeping when cf bits editor is closed */
    void slotConfigBitsEditorClosed();
private: // Private attributes
    /** New project widget */
    NewProject * newproject;
    /** Last selected item */
    QTreeWidgetItem *currentItem ;
    /** Project list view */
    QTreeWidget *projView ;
    /** Root of Project tree */
    QTreeWidgetItem  *projectTree;
    Project *project ;
    /** Link to main window */
    PikMain *main_window ;
    /** Project editor, non null when open */
    ProjectEditorWid * projecteditor;
    /** Config bits editor, non null when open */
    ConfigBitsWizard *configbits_editor ;
    /** Timer for triggering of source code parsing */
    QTimer *entryPointTimer ;
private: // Private methods
    /** Make an empty popup menu with "title" header */
    QMenu *makePopup ( const QString& title, QWidget  *parent );
    /** insert a new absolute filename - returns true if OK, false if already exists */
    bool insertAbsFile ( const QString &fname ) ;
    /** remove all children of list */
    void removeChildren (  QTreeWidgetItem *list ) ;
    /** open the document  which corresponds to this item -
                        returns adress of the editor, or 0 if doc is not found  */
    QBEditor * openEditor (  QTreeWidgetItem  *item ) ;

private slots: // Private slots
    /** No descriptions */
    void slotProjectElementClicked ( QTreeWidgetItem *,int );
public: // Public attributes
};

#endif
